library(Matrix)
library(tseries)
library(ada)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")

#adaboost
train_adaboost <- function(splitid, nfeat, iter = 20, loss = "exponential") {
	testidx = which(splits[,splitid]==1)
	trainidx = which(splits[,splitid]==0)
	print(splitid)
	
	trmat <- as.matrix(nfeat[trainidx,])
	tsmat <- as.data.frame(nfeat[testidx,])
	
	set.seed(0)
	predictions <-  matrix(0, nrow = length(testidx), ncol = length(event.ids))
	for(eventid in 1:length(event.ids)) {
		trlabels = labels[trainidx, eventid]
		set.seed(0) 
		model <- ada(x = trmat, y = factor(trlabels), loss=loss, iter = iter)
		pred<- predict(model, tsmat, type = "prob")
		tsprediction <- pred[,2]
		predictions[,eventid] = tsprediction
	}
	return(predictions)
}



event.ids <- paste("E",1:16,sep = "")
idlist = as.character(read.table("data/olympic.idlist")[,1])
labels = read.matrix("data/olympic.labels")
splits = as.matrix(read.matrix("data/olympic.splits"))

nfeat = read.matrix("data/olympic.fet.pca")
#this is how nfeat is generated
#feat = read.matrix("data/dtraj.matrix")
#dim = nrow(feat)
#pca = prcomp(feat, center = TRUE)
#nfeat = pca$x
#write(t(nfeat), file="data/olympic.fet.pca", ncolumns = ncol(nfeat))

#tuning the parameters
lu = "olympic+adaboost"
iter = c(10,20,50,80)
loss = c("exponential","logistic")

para.table = expand.grid(iter = iter, loss = loss)
para.map = matrix(0, nrow = nrow(para.table), ncol = length(event.ids)) 

for(i in 1:nrow(para.table)) {
	predictions = train_adaboost(1, nfeat, iter=para.table[i,1], loss = para.table[i,2])
	testlabels = labels[which(splits[,1]==1),]
	para.map[i,] = evaluate.map(predictions, testlabels)
}
cbind(para.table, para.map)

#you should be able to see the results like
#  iter        loss  para.map
#1   10 exponential                0.5715955
#2   20 exponential                0.5990740
#3   50 exponential                0.6649840
#4   80 exponential                0.6913551
#5   10    logistic                0.5491246
#6   20    logistic                0.6271792
#7   50    logistic                0.6760894
#8   80    logistic                0.6924610


predictions = train_adaboost(1, nfeat, iter=80, loss = "logistic")
testlabels = labels[which(splits[,1]==1),]
evaluate.map(predictions, testlabels)
