library(Matrix)
library(tseries)
library(randomForest)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")

#random forest
train_randforest <- function(splitid, nfeat, classwt = c(0.06,1), ntree=16, mtry = 16) {
	testidx = which(splits[,splitid]==1)
	trainidx = which(splits[,splitid]==0)
	print(splitid)
	
	trmat <- as.matrix(nfeat[trainidx,])
	tsmat <- as.matrix(nfeat[testidx,])
	
	set.seed(0)
	predictions <-  matrix(0, nrow = length(testidx), ncol = length(event.ids))
	for(eventid in 1:length(event.ids)) {
		trlabels = labels[trainidx, eventid]
		set.seed(0) 
		model <- randomForest(x = trmat, y = factor(trlabels), classwt = c(0.2,1), ntree=ntree, mtry=mtry)
		pred<- predict(model, tsmat, type="vote")
		classes = attributes(pred)$dimnames[[2]]
		posclassidx = which(classes=="1")
		tsprediction = as.numeric(pred[,posclassidx])
		predictions[,eventid] = tsprediction
	}
	return(predictions)
}



event.ids <- paste("E",1:16,sep = "")
idlist = as.character(read.table("data/olympic.idlist")[,1])
labels = read.matrix("data/olympic.labels")
splits = as.matrix(read.matrix("data/olympic.splits"))

nfeat = read.matrix("data/olympic.fet.pca")
#this is how nfeat is generated
#feat = read.matrix("data/dtraj.matrix")
#dim=nrow(feat)
#pca = prcomp(feat, center = TRUE)
#nfeat = pca$x
#write(t(nfeat), file="data/olympic.fet.pca", ncolumns = ncol(nfeat))


#tuning the parameters
lu = "olympic+randomforest"
ntree = c(128,512,1024)
mtry = c(32,64,128)
para.table = expand.grid(ntree = ntree, mtry = mtry)
para.map = matrix(0, nrow = nrow(para.table), ncol = length(event.ids)) 

for(i in 1:nrow(para.table)) {
	predictions = train_randforest(1, nfeat, ntree=para.table[i,1], mtry = para.table[i,2])
	testlabels = labels[which(splits[,1]==1),]
	para.map[i,] = evaluate.map(predictions, testlabels)
}
cbind(para.table, para.map)
cbind(para.table,apply(para.map,1,mean))

#you should be able to see the results like
#    ntree mtry              para.map
#1   128   32                0.5389921
#2   512   32                0.5523295
#3  1024   32                0.5502371
#4   128   64                0.5708140
#5   512   64                0.6028684
#6  1024   64                0.5952584
#7   128  128                0.5943222
#8   512  128                0.6332013
#9  1024  128                0.6307709


predictions = train_randforest(1, nfeat, ntree=512, mtry = 128)
testlabels = labels[which(splits[,1]==1),]
aps = evaluate.map(predictions, testlabels)
print(mean(aps))