library(Matrix)
library(tseries)
library(kernlab)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")
source("../selfpace_lib.r")

selfpaced.single <- function(splitid, initialvals = NULL, pos.stepsize = 1, neg.stepsize = 100, C = 0.1) {
	print(splitid)
	trainidx = which(splits[,splitid]==0)
	testidx = which(splits[,splitid]==1)

	#the list storing the result for each iteration
	valpredmatrix = list()
	evlpredmatrix = list()	
	for(iter in 1:50) {
		valpredmatrix[[iter]] = matrix(0, nrow = length(trainidx), ncol = length(event.ids))
		evlpredmatrix[[iter]] = matrix(0, nrow = length(testidx), ncol = length(event.ids))
	}
	
	#the statring values
	initial.posidx = list()
	initial.negidx = list()
	
	#the selected positive and negative samples for all iterations
	iters.posidx = list()	
	iters.negidx = list()
	
	
	for(eventid in 1:length(event.ids)) {#for each event
		#validation set is different from the training set
		dev_labels = labels[trainidx, eventid]
		
		this.training.posidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==1))
		this.training.negidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==0))
		
		pos.stepsize = pos.stepsize
		neg.stepsize = neg.stepsize
		
		#self-pace parameters
		pos.lambda1_rank = pos.stepsize
		neg.lambda1_rank = neg.stepsize
		
		
		if(is.null(initialvals)) {
			#when the starting value is missing, use randomly sampled positives and negatives.
			this.training.posidx = sample(this.training.posidx, pos.lambda1_rank)
			this.training.negidx = sample(this.training.negidx, neg.lambda1_rank)
		} else {
			#when the starting value is set, use it.
			this.training.posidx = initialvals$pos[[eventid]]
			this.training.negidx = initialvals$neg[[eventid]]
		}
		
		#store starting values
		initial.posidx[[eventid]] = this.training.posidx
		initial.negidx[[eventid]] = this.training.negidx
		

		iters.posidx[[eventid]] = list()
		iters.negidx[[eventid]] = list()
		
		for(iter in 1:50) {
			print(paste("action", eventid, "iter ", iter))
			print(paste("-----selected neg=", length(this.training.negidx), " pos=", length(this.training.posidx), sep = ""))
			
			#store the posidx and negidx used in this iteration
			iters.posidx[[eventid]][[iter]] = this.training.posidx
			iters.negidx[[eventid]][[iter]] = this.training.negidx
			
			thistraining.idx = c(this.training.posidx, this.training.negidx)
			thistraining.labels = c(replicate(length(this.training.posidx),1), replicate(length(this.training.negidx),0))

			if(length(which((labels[thistraining.idx,eventid] == thistraining.labels)==FALSE))!=0) {
				#check whether the labels are consistent (for debug purposes)
				print(paste("Warning: incosistent labels found, please check: action", eventid, "iter ", iter))
				return
			}
			
			#off-the-shelf SVM training
			thistrmat = as.kernelMatrix(kernels[thistraining.idx,thistraining.idx])
			model = ksvm(x=thistrmat, y=thistraining.labels, type = "C-svc", C = C)
		
			#SVM validates on the training set
			thisdevmat = as.kernelMatrix(kernels[trainidx,thistraining.idx[SVindex(model)]])
			valprediction = predict(model, thisdevmat, type = "decision")
			trprediction = valprediction
			
			#SVM validates on the test set
			thistsmat = as.kernelMatrix(kernels[testidx,thistraining.idx[SVindex(model)]])
			tsprediction = predict(model, thistsmat, type = "decision")
			
			#self-paced learning
			selectedidx = selfpace1.rank(trprediction, dev_labels, pos.lambda1_rank, neg.lambda1_rank)

			#change the parameter accoding to the step size
			pos.lambda1_rank = pos.lambda1_rank + pos.stepsize
			neg.lambda1_rank = neg.lambda1_rank + neg.stepsize
			
			#store the prediction for this iteration
			valpredmatrix[[iter]][,eventid] = as.numeric(valprediction)
			evlpredmatrix[[iter]][,eventid] = tsprediction
			
			#set the selected positive and negative for the next iteration.
			this.training.negidx = trainidx[unique(selectedidx$neg)]
			this.training.posidx = trainidx[unique(selectedidx$pos)]			
		}
	}
	initialvals = list("pos" = initial.posidx, "neg" = initial.negidx)
	results <- list("valpredmatrix" = valpredmatrix, "evlpredmatrix" = evlpredmatrix, "initialvals" = initialvals, "iterposidx" = iters.posidx, "iternegidx" = iters.negidx)
	return(results)
}

evaluate.val <- function(splitid, package, devidx, mode = "aps", bestiters=NULL) {
	#calculate the validation set aps based on the given validation set
	
	#the training idx
	trainingidx = which(splits[,splitid]==0)
	
	#final average precisions
	aps = matrix(0, nrow=1, ncol = length(event.ids))
		
	#testset idx and labels
	testidx = which(splits[,splitid]==1)
	testlabels = labels[testidx,]
	
	#development idx in training set
	devidxintraining = unlist(lapply(devidx, function(x) {which(trainingidx==x)}))
	traininglabels = labels[devidx,]

	
	devaps = matrix(0, nrow=50, ncol=length(event.ids))	
	testaps = matrix(0, nrow=50, ncol=length(event.ids))

	#calculate map for each iteration
	for(iterid in 1:50) {
		testaps[iterid,] = evaluate.map(package$evlpredmatrix[[iterid]], testlabels)
		devaps[iterid,] = evaluate.map(package$valpredmatrix[[iterid]][devidxintraining,], traininglabels)
	}
	
	#find the best iteration based on development aps
	if(is.null(bestiters)) bestiters = findbestiter2(devaps)
	print("##########best iter ###############")
	print(bestiters)
	
	#caluclta the final aps at the best iteration
	for(eventid in 1:length(event.ids)) {
		aps[splitid, eventid] = testaps[bestiters[eventid],eventid]
	}
	
	if(mode == "aps") {
		return(t(aps))
	} else if( mode == "bestiters") {
		return(bestiters)
	} else if( mode == "devevl") {
		return(list("devaps" = devaps, "evlaps" = testaps))
	} else if( mode == "aps+bestiters") {
		return(list(t(aps),bestiters))
	}
}



event.ids <- paste("E",1:16,sep = "")
kernels = read.matrix("data/dev.matrix")
kernels = kernels * 2	#our tool divides pre-calculated kernel by 2 and here we get it back

idlist = as.character(read.table("data/olympic.idlist")[,1])
labels = read.matrix("data/olympic.labels")
splits = as.matrix(read.matrix("data/olympic.splits"))


#tuning the parameters
lu = "olympic+spl"
posstepsize = c(1,2,3,4,5,6)
negstepsize = c(8,9,10,11,12,15)

para.table = expand.grid(posstepsize = posstepsize, negstepsize = negstepsize)
para.runs = list()

for(i in 1:nrow(para.table)) {
	#get the prediction
	package = selfpaced.single(1, pos.stepsize = para.table[i,1], neg.stepsize = para.table[i,2])
	para.runs[[i]] = package
}


#evaluate the prediction
devidx = read.table("olympic_devidx.list")[,1]	#load the validation set idx
para.map = replicate(length(para.runs), 0)
para.iters = list()
for(runid in 1:length(para.runs)) {
	package = para.runs[[runid]]
	result = evaluate.val(1, package, devidx, mode = "aps+bestiters")
	para.map[runid] = mean(result[[1]])
	para.iters[[runid]] = result[[2]]
}
which(para.map == max(para.map))

print(cbind(para.table, para.map))
print(max(para.map))



#######################################################################
#######################################################################
### if you cannot get the map in our paper                          ###
### probably due to the randomness in starting values/clustering    ###
### try the following code                     						###
#######################################################################
#######################################################################
aps = replicate(length(event.ids),0)
#the samples used in the final models
sampleset = read.matrix("olympic.spl.besttrainingset.matrix")
for(eventid in 1:length(event.ids)) {
	thistraining.idx = which(sampleset[,eventid] != -1059)
	thistraining.labels = labels[thistraining.idx,eventid]
	testidx = which(splits[,1]==1)
	
	thistrmat = as.kernelMatrix(kernels[thistraining.idx,thistraining.idx])
	model = ksvm(x=thistrmat, y=thistraining.labels, type = "C-svc", C = 0.1)
		
	thistsmat = as.kernelMatrix(kernels[testidx,thistraining.idx[SVindex(model)]])
	tsprediction = predict(model, thistsmat, type = "decision")
	
	aps[eventid] = ap(cbind(labels[testidx,eventid], tsprediction))
}
print(cbind(1:length(aps), aps))
print(mean(aps))

