library(Matrix)
library(tseries)
library(kernlab)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")
source("../selfpace_lib.r")



selfpaced.diversity.single <- function(splitid, initialvals = NULL, diversity_ratio=0.01, clustermembership, pos.stepsize = 1, neg.stepsize = 100, C = 0.1) {
	print(splitid)
	trainidx = which(splits[,splitid]==0)
	testidx = which(splits[,splitid]==1)
	#devidx = devsets[[splitid]]

	#this matrix stores the final result. Each for a iteration for every event.
	valpredmatrix = list()
	evlpredmatrix = list()	
	for(iter in 1:50) {
		valpredmatrix[[iter]] = matrix(0, nrow = length(trainidx), ncol = length(event.ids))
		evlpredmatrix[[iter]] = matrix(0, nrow = length(testidx), ncol = length(event.ids))
	}
	
	temp = clustermembership[,1]
	dev_membership = unlist(lapply(trainidx, function(x) {return(which(temp==x))}))
	dev_membership = clustermembership[dev_membership,2]
	
	initial.posidx = list()
	initial.negidx = list()
	
	iters.posidx = list()	
	iters.negidx = list()
	
	for(eventid in 1:length(event.ids)) {
		dev_labels = labels[trainidx, eventid]
		val_labels = dev_labels
		
		this.training.posidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==1))
		this.training.negidx = intersect(which(splits[,splitid]==0), which(labels[, eventid]==0))
		
	
		pos.stepsize = pos.stepsize
		neg.stepsize = neg.stepsize
		
		pos.lambda1_rank = pos.stepsize
		neg.lambda1_rank = neg.stepsize
		
		
		if(is.null(initialvals)) {
			this.training.posidx = sample(this.training.posidx, pos.lambda1_rank)
			this.training.negidx = sample(this.training.negidx, neg.lambda1_rank)
		} else {
			this.training.posidx = initialvals$pos[[eventid]]
			this.training.negidx = initialvals$neg[[eventid]]
		}
		
		initial.posidx[[eventid]] = this.training.posidx
		initial.negidx[[eventid]] = this.training.negidx
		
		
		iters.posidx[[eventid]] = list()
		iters.negidx[[eventid]] = list()
		
		for(iter in 1:50) {
			print(paste("event", eventid, "iter ", iter))
			print(paste("-----selected neg=", length(this.training.negidx), " pos=", length(this.training.posidx), sep = ""))
			
			#record the posidx and negidx used in this iteration
			iters.posidx[[eventid]][[iter]] = this.training.posidx
			iters.negidx[[eventid]][[iter]] = this.training.negidx
			
			thistraining.idx = c(this.training.posidx, this.training.negidx)
			thistraining.labels = c(replicate(length(this.training.posidx),1), replicate(length(this.training.negidx),0))

			if(length(which((labels[thistraining.idx,eventid] == thistraining.labels)==FALSE))!=0) {
				#check whether the labels are consistent (for debug purposes)
				print(paste("Warning: incosistent labels found, please check: action", eventid, "iter ", iter))
				return
			}
			
			#off-the-shelf SVM training
			thistrmat = as.kernelMatrix(kernels[thistraining.idx,thistraining.idx])
			model = ksvm(x=thistrmat, y=thistraining.labels, type = "C-svc", C = C)
		
			thisdevmat = as.kernelMatrix(kernels[trainidx,thistraining.idx[SVindex(model)]])
			
			valprediction = predict(model, thisdevmat, type = "decision")
			trprediction = valprediction
			
			thistsmat = as.kernelMatrix(kernels[testidx,thistraining.idx[SVindex(model)]])
			tsprediction = predict(model, thistsmat, type = "decision")
			
			
			#self-paced learning
			selectedidx = selfpace2.rank(trprediction, dev_labels, dev_membership, pos.lambda1_rank, neg.lambda1_rank, diversity_ratio = diversity_ratio)
			
			#change the parameter accoding to the step size
			pos.lambda1_rank = pos.lambda1_rank + pos.stepsize
			neg.lambda1_rank = neg.lambda1_rank + neg.stepsize
			

			valpredmatrix[[iter]][,eventid] = as.numeric(valprediction)
			evlpredmatrix[[iter]][,eventid] = tsprediction
			
			this.training.negidx = trainidx[unique(selectedidx$neg)]
			this.training.posidx = trainidx[unique(selectedidx$pos)]
			
		}
	}
	initialvals = list("pos" = initial.posidx, "neg" = initial.negidx)
	results <- list("valpredmatrix" = valpredmatrix, "evlpredmatrix" = evlpredmatrix, "initialvals" = initialvals, "iterposidx" = iters.posidx, "iternegidx" = iters.negidx)
	return(results)
}

evaluate.val <- function(splitid, package, devidx, mode = "aps", bestiters=NULL) {
	trainingidx = which(splits[,splitid]==0)
	
	aps = matrix(0, nrow=1, ncol = length(event.ids))
		
	testidx = which(splits[,splitid]==1)
	testlabels = labels[testidx,]
	traininglabels = labels[devidx,]
	
	
	devidxintraining = unlist(lapply(devidx, function(x) {which(trainingidx==x)}))
	
	devaps = matrix(0, nrow=50, ncol=length(event.ids))
	testaps = matrix(0, nrow=50, ncol=length(event.ids))

	for(iterid in 1:50) {
		testaps[iterid,] = evaluate.map(package$evlpredmatrix[[iterid]], testlabels)
		devaps[iterid,] = evaluate.map(package$valpredmatrix[[iterid]][devidxintraining,], traininglabels)
	}
	
	
	if(is.null(bestiters)) bestiters = findbestiter1(devaps)
	print("##########best iter ###############")
	print(bestiters)
	
	
	for(eventid in 1:length(event.ids)) {
		aps[splitid, eventid] = testaps[bestiters[eventid],eventid]
	}
	
	if(mode == "aps") {
		return(t(aps))
	} else if( mode == "bestiters") {
		return(bestiters)
	} else if( mode == "devevl") {
		return(list("devaps" = devaps, "evlaps" = testaps))
	} else if( mode == "aps+bestiters") {
		return(list(t(aps),bestiters))
	}
}



event.ids <- paste("E",1:16,sep = "")
kernels = read.matrix("data/dev.matrix")
kernels = kernels * 2	#our tool divides pre-calculated kernel by 2 and here we get it back
idlist = as.character(read.table("data/olympic.idlist")[,1])
labels = read.matrix("data/olympic.labels")
splits = as.matrix(read.matrix("data/olympic.splits"))

lu = "olympic+spld"
#load the low dimensional features
nfeat = read.matrix("data/olympic.fet.pca")

#kmeans
trainingidx = which(splits[,1] ==0)
clusters = kmeans(nfeat[trainingidx,], 128, iter.max = 1000)
clustermembership = cbind(trainingidx, clusters$cluster)

#specc
#library(kernlab)
#clusters = specc(feat[trainingidx,], 64)
#clustermembership = cbind(trainingidx, clusters[1:length(idlist)])


#tuning the parameters
posstepsize = c(1,2,3)
negstepsize = c(8,10,15)
diversityratio = c(0.001,0.01,0.1,1)

para.table = expand.grid(posstepsize = posstepsize, negstepsize = negstepsize, diversityratio=diversityratio)
para.runs = list()

for(i in 1:nrow(para.table)) {
	#get the prediction
	package = selfpaced.diversity.single(1, pos.stepsize = para.table[i,1], neg.stepsize = para.table[i,2], diversity_ratio=para.table[i,3], clustermembership=clustermembership)
	para.runs[[i]] = package
}
cbind(para.table)


#evaluate the runs
devidx = read.table("olympic_devidx.list")[,1]	#load the validation set
para.map = replicate(length(para.runs), 0)
para.iters = list()
for(runid in 1:length(para.runs)) {
	package = para.runs[[runid]]
	result = evaluate.val(1, package, devidx, mode = "aps+bestiters")
	para.map[runid] = mean(result[[1]])
	para.iters[[runid]] = result[[2]]
}
