library(Matrix)
library(tseries)
library(kernlab)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")
source("../selfpace_lib.r")



event.ids <- paste("E",1:16,sep = "")
kernels = read.matrix("data/dev.matrix")
kernels = kernels * 2	#our tool divides pre-calculated kernel by 2 and here we get it back

idlist = as.character(read.table("data/olympic.idlist")[,1])
labels = read.matrix("data/olympic.labels")
splits = as.matrix(read.matrix("data/olympic.splits"))


#tuning the parameters
lu = "olympic+spld"



#Note if you cannot get 93.1% using this code this is proabaly due to the
#randomness in starting values, clustering algorithm and the selection of
#best iteration. If you keep tuning parameters you will finally get the
#solution.

#For this dataset, since it is easy, there are lots of iteration having the
#same perfect development ap i.e. 100%. But they have different test ap. 
#So the randomness in breaking a tie affects the final results. 
#In our experiments we thoroughly tuned the parameters and executed 
#the code many times and then selected the one with the best performance.

#you can use the following code to reproduce our results
aps = replicate(length(event.ids),0)
#the samples used in the final models
sampleset = read.matrix("olympic.spld.besttrainingset.matrix")
for(eventid in 1:length(event.ids)) {
	thistraining.idx = which(sampleset[,eventid] != -1059)
	thistraining.labels = labels[thistraining.idx,eventid]
	testidx = which(splits[,1]==1)
	
	thistrmat = as.kernelMatrix(kernels[thistraining.idx,thistraining.idx])
	model = ksvm(x=thistrmat, y=thistraining.labels, type = "C-svc", C = 0.1)
		
	thistsmat = as.kernelMatrix(kernels[testidx,thistraining.idx[SVindex(model)]])
	tsprediction = predict(model, thistsmat, type = "decision")
	
	aps[eventid] = ap(cbind(labels[testidx,eventid], tsprediction))
}
print(cbind(1:length(aps), aps))
print(mean(aps))